      program display
C----
C---- (C) Bert de Groot, Goettingen 2000
C----
C---- creates a score map based on a number of rottrans runs      
C----
      implicit none

      include 'params.f'

      integer i,j,k,l,nat,atnr(maxat),resnr(maxat),nres,a,
     &     nx,ny,nz,ngrid(3),orig(3),ntit,id(maxat),iargc,
     &     dim,dim2,n,nrun,gvflen,pdblen,scorlen,nscore
      
      character*5 atnam(maxat),resnam(maxat)

      real pi,x(maxat*3),xav(3),alpha,beta,gamma,ca,sa,ri,rj,rk,
     &     cb,sb,cg,sg,xr(maxat),yr(maxat),zr(maxat),
     &     space(3),cell(6),xx(maxat),yy(maxat),zz(maxat),ru,rv,rw,
     &     calden(maxmap),sumden(maxmap),den1(maxat),den2(maxat),
     &     a1(10),a2(10),b1(10),b2(10),score,Bfac(maxat),occ(maxat)

      character*80 sdum,pdbnam,scornam

      character*4 cdum1,cdum2

      character*1 chainid(maxat)

      character*255 buf

      logical gvfisf
C----
C---- initialise
C----
      if (iargc().ne.6) then
         write(0,'(a,a)') 'Usage: display in.map inscore nscore ',
     &        'inpdb nfiles out.map'
         write(0,*)
         write(0,'(a,a)') 'Generates a scoremap "out.map" from ',
     &        'output of multiple "rottrans" runs.'
         write(0,'(a,a)') 'Well-fitting fragments will show up ',
     &        'in the map and hopefully simplify interpretation.'
         write(0,'(a,a)') '"out.map" will have the same parameters ',
     &        'as "in.map".'
         write(0,'(a,a)') '"inscore" and "inpdb" are basic file ',
     &        'names to which a number (1..nfiles) and ' 
         write(0,'(a,a)') 'suffix will be added (none and .pdb ',
     &        'respectively).'
         write(0,'(a,a)') '"nscore" is the number of scores per file',
     &        ' that will be taken into account.'
         write(0,'(a,a)') '"in.map" has to be x-plor formatted format',
     &        ' for now.'
         stop
      endif
      pi=4.0*atan(1.0)
      do i=1,maxmap
         sumden(i)=0.0
      enddo
C----
C---- read map header for params
C----
      call getarg(1,sdum)
      call getarg(1,sdum)
      call readxplormapheader(sdum,cell,nx,ny,nz,space,orig,ngrid)
      if (nx*ny*nz.gt.maxmap) then
         write(0,'(a)') 'Error - nx,ny,nz too large.'
         write(0,'(a,i10)') 'Increase maxmap to at least',nx*ny*nz
         stop
      endif
C----
C---- loop over the number of rottrans runs
C----
      call getarg(2,scornam)
      call gvstcp(scornam)
      scorlen=gvflen(scornam)
      call getarg(3,sdum)
      read(sdum,*) nscore
      call getarg(4,pdbnam)
      call gvstcp(pdbnam)
      pdblen=gvflen(pdbnam)
      call getarg(5,sdum)
      read (sdum,*) nrun
      do n=1,nrun
         write(6,'(a,i5)') 'processing run ',n
C----
C---- read input structure
C----
         xav(1)=0.0
         xav(2)=0.0
         xav(3)=0.0
         write(sdum,*) pdbnam(1:pdblen),n,'.pdb'
         call gvstcp(sdum)
         if (gvfisf(sdum)) then
            open(1,file=sdum,status='old')
         else
            call gvstcp(sdum)
            write(0,'(a,a)') 'Error - file not found : ',
     &           sdum(1:gvflen(sdum))
            stop
         endif
         nat=0
         nres=1
         cdum2='xxxx'
 50      continue
         read(1,'(a80)',err=60,end=70) sdum
         if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 50
         nat=nat+1
         if (nat.gt.maxat) then
            write(0,'(a)') 'Error - MAXAT exceeded'
            stop
         endif
         read(sdum,1000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &        x(3*nat-2),x(3*nat-1),x(3*nat),occ(nat),Bfac(nat)
         if (nat.gt.1) then
            if (cdum1.ne.cdum2) nres=nres+1
         endif
         if (atnam(nat).ne.'     ') then
            call gvsshr(atnam(nat),5)
         else
            write(0,*) 'Encountered empty atom name for atom number',
     &           nat
            stop
         endif
         resnr(nat)=nres
         atnr(nat)=nat
         cdum2=cdum1
         goto 50
 60      write(0,'(a)') 'Error while reading structure'
         stop
 70      continue
         if (nat.eq.0) then
            write(0,'(a,a)') 'Unexpected end of file while reading ',
     &           'structure'
            stop
         endif
         close(1)

         do i=1,nat
            xav(1)=xav(1)+x(3*i-2)
            xav(2)=xav(2)+x(3*i-1)
            xav(3)=xav(3)+x(3*i)
         enddo
         close(1)
         
         xav(1)=xav(1)/real(nat)
         xav(2)=xav(2)/real(nat)
         xav(3)=xav(3)/real(nat)
         
         do i=1,nat
            xx(i)=x(3*i-2)-xav(1)
            yy(i)=x(3*i-1)-xav(2)
            zz(i)=x(3*i)-xav(3)
         enddo
C----
C---- get the atomic scattering factors etc
C----
         call getatomscat(a1,a2,b1,b2,atnam,nat,Bfac,id,den1,den2)
         do i=1,nat
            den1(i)=den1(i)*occ(i)
            den2(i)=den2(i)*occ(i)
         enddo
C----
C---- read the highest scoring orientations from file and generate them
C----
         write(sdum,*) scornam(1:scorlen),n
         call gvstcp(sdum)
         if (gvfisf(sdum)) then
            open(1,file=sdum,status='old')
         else
            call gvstcp(sdum)
            write(0,'(a,a)') 'Error - file not found : ',
     &           sdum(1:gvflen(sdum))
            stop
         endif
         do a=1,nscore
            read(1,*,end=100) ri,rj,rk,ru,rv,rw,score
            gamma=pi*ri/180.0
            cg=cos(gamma)
            sg=sin(gamma)
            beta=pi*rj/180.0
            cb=cos(beta)
            sb=sin(beta)
            alpha=pi*rk/180.0
            ca=cos(alpha)
            sa=sin(alpha)
            do l=1,nat
               xr(l)=xx(l)*(cg*cb*ca-sg*sa)
               xr(l)=xr(l)-yy(l)*(sg*cb*ca+cg*sa)
               xr(l)=xr(l)+zz(l)*(sb*ca)
               xr(l)=xr(l)+(ru+real(orig(1)-1))*space(1)
               yr(l)=xx(l)*(cg*cb*sa+sg*ca)
               yr(l)=yr(l)-yy(l)*(sg*cb*sa-cg*ca)
               yr(l)=yr(l)+zz(l)*(sb*sa)
               yr(l)=yr(l)+(rv+real(orig(2)-1))*space(2)
               zr(l)=xx(l)*(-cg*sb)
               zr(l)=zr(l)+yy(l)*(sg*sb)
               zr(l)=zr(l)+zz(l)*cb
               zr(l)=zr(l)+(rw+real(orig(3)-1))*space(3)
            enddo
            if (score.gt.-999.) then
               call calc_dens(den1,den2,id,a1,a2,b1,b2,xr,yr,zr,Bfac,
     &              nat,calden,space,orig,nx,ny,nz,maxat,maxmap)
C----
C---- apply score as weight and add to output map
C----
               do i=1,maxmap
                  if (calden(i).gt.1.0e-5) 
     &                 sumden(i)=sumden(i)+calden(i)*score
               enddo
            endif
         enddo
 100     continue
         close(1)
      enddo
C----
C---- write out the score map
C----
      write(6,'(a)') 'Writing output map...'
      call getarg(6,sdum)
      open(1,file=sdum,status='unknown')
      write(1,*)
      write(1,'(i8)') 1
      write(1,'(a)') 'REMARKS Display map generated by display'
      write(1,'(9i8)') 
     &  ngrid(1),orig(1),nx+orig(1)-1,
     &  ngrid(2),orig(2),ny+orig(2)-1,
     &  ngrid(3),orig(3),nz+orig(3)-1
      write(1,'(1p,6e12.5)') (cell(i),i=1,6)
      write(1,'(a)') 'ZYX'
      dim=nx
      dim2=nx*ny
      do k=1,nz
         write(1,'(i8)') k
         write(1,'(6e12.5)') ((sumden((k-1)*dim2+(j-1)*dim+i),
     &        i=1,nx),j=1,ny)
      enddo
      close(1)

 1000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,2f6.2)

      end
      
