      program mapstat
C----
C---- (C) Bert de Groot, Goettingen 2000
C----
C---- calculates various global and local (around atoms) map statistics
C----
      implicit none

      include 'params.f'

      integer i,j,k,n,nat,atnr(maxat),resnr(maxat),nres,
     &     nx,ny,nz,ngrid(3),orig(3),ntit,id(maxat),iargc,ind,
     &     dim,dim2,gvflen,imin,imax,jmin,jmax,kmin,kmax,
     &     ind3,np,idd(maxat),natd,ppi(maxgrid),
     &     ppj(maxgrid),ppk(maxgrid)

      character*5 atnam(maxat),resnam(maxat),dum1

      character*4 cdum1,cdum2

      character*1 chainid(maxat)
      
      real pi,space(3),cell(6),xx(maxat),yy(maxat),zz(maxat),
     &     calden(maxmap),den1(maxat),den2(maxat),a1(9),avexp,sigexp,
     &     a2(9),b1(9),b2(9),expden(maxmap),avcal,sigcal,
     &     covar,xd(maxat),yd(maxat),zd(maxat),den1d(maxat),occd(maxat),
     &     den2d(maxat),shift,scale,sh2,sc2,calden2(maxmap),occ(maxat),
     &     avocc,scdum,shdum,Bfac(maxat),Bfacd(maxat)

#     if defined (OPTIMISE_OCCUPANCIES)
         real ppc(maxgrid,maxat)
#     else
         real ppc(1,maxat)
#     endif

      real*8 serr

      character*80 sdum

      character*255 buf

      logical gvfisf
C----
C---- initialise
C----
      if (iargc().ne.2) then
         write(0,'(a,a)') 'Usage: mapstat in.map in.pdb'
         write(0,*)
         write(0,'(a,a)') 'The program performs all kinds of ',
     &        'statistics over the map:'
         write(0,'(a,a)') 'Overall statistics and statistics over a ',
     &        'mask built from "in.pdb"'
         write(0,'(a,a)') '"in.map" has to be x-plor formatted format',
     &        ' for now.'
         stop
      endif
      pi=4.0*atan(1.0)
      scdum=1.0
      shdum=0.0
C----
C---- read map
C----
      call getarg(1,sdum)
      call readxplormapheader(sdum,cell,nx,ny,nz,space,orig,ngrid)
      dim=nx
      dim2=nx*ny
      if (nx*ny*nz.gt.maxmap) then
         write(0,'(a)') 'Error - nx,ny,nz too large.'
         write(0,'(a,i10)') 'Increase maxmap to at least',nx*ny*nz
         stop
      endif
      call readxplormap(sdum,nx,ny,nz,space,orig,expden)
C----
C---- get overall statistics
C----
      write(6,'(a)') 'Overall map statistics'
      avexp=0.0
      sigexp=0.0
      do i=1,nx*ny*nz
         avexp=avexp+expden(i)
         sigexp=sigexp+expden(i)**2
      enddo
      avexp=avexp/real(nx*ny*nz)
      sigexp=sqrt(sigexp/real(nx*ny*nz)-avexp**2)
      write(6,'(a,2e14.6)') 'Average, sigma of experimental map ',
     &     avexp,sigexp
      write(6,*)
C----
C---- read the input structure
C----
      call getarg(2,sdum)
      if (gvfisf(sdum)) then
         open(1,file=sdum,status='old')
      else
         call gvstcp(sdum)
         write(0,'(a,a)') 'Error - file not found : ',
     &        sdum(1:gvflen(sdum))
         stop
      endif
      nat=0
      nres=1
      cdum2='xxxx'
 50   continue
      read(1,'(a80)',err=60,end=70) sdum
      if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 50
      nat=nat+1
      if (nat.gt.maxat) then
         write(0,'(a)') 'Error - MAXAT exceeded'
         stop
      endif
      read(sdum,2000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &     xx(nat),yy(nat),zz(nat),occ(nat),Bfac(nat)
      if (nat.gt.1) then
         if (cdum1.ne.cdum2) nres=nres+1
      endif
      if (atnam(nat).ne.'     ') then
         call gvsshr(atnam(nat),5)
      else
         write(0,*) 'Encountered empty atom name for atom number',nat
         stop
      endif
      resnr(nat)=nres
      atnr(nat)=nat
      cdum2=cdum1
      goto 50
 60   write(0,'(a)') 'Error while reading input structure'
      stop
 70   continue
      if (nat.eq.0) then
         write(0,'(a,a)') 'Unexpected end of file while reading ',
     &        'input structure'
         stop
      endif
      close(1)
C----
C---- get the atomic scattering factors etc
C----
      call getatomscat(a1,a2,b1,b2,atnam,nat,Bfac,id,den1,den2)
c      call resoscat(3.5,a1,a2,b1,b2,Bover,6)
C----
C---- calculate the densities
C----
      call calc_dens3(den1,den2,id,a1,a2,b1,b2,xx,yy,zz,nat,imin,imax,
     &     jmin,jmax,kmin,kmax,np,calden,space,orig,nx,ny,nz,maxat,
     &     maxmap,maxgrid,ppi,ppj,ppk,ppc,occ,Bfac,scdum,shdum)

      write(6,'(a)') 'Mask statistics'
      write(6,'(a,i5)') 'Number of atoms:',nat
C----
C---- evaluate optimal scale and shift for calculated densities
C----
      do i=1,nx*ny*nz
         calden2(i)=calden(i)
      enddo
c      call getscale(calden,expden,imin,imax,jmin,jmax,kmin,kmax,
c     &     shift,scale,maxmap,dim,dim2)
      scale=1.0
      shift=0.0
      write(6,'(a)') 'Iterating to find optimal scale and shift...'
      do i=1,100
         call getscale2(calden,expden,imin,imax,jmin,jmax,kmin,kmax,
     &        sh2,sc2,calden2,maxmap,dim,dim2)
         scale=scale*sc2
         shift=sc2*shift+sh2
      enddo
      if ((abs(sc2-1.0)).gt.0.01) then
         write(6,'(a)') '*********************************************'
         write(6,'(a)') 'WARNING - scale calculation did NOT converge.' 
         write(6,'(a)') 'Reported scale and shift are unreliable!!!'
         write(6,'(a)') '*********************************************'
      endif
C---- 
C---- calculate mask statistics between calculated and experimental map
C----
      open(1,file='calexp.den',status='unknown')
      avexp=0.0
      sigexp=0.0
      avcal=0.0
      sigcal=0.0
      covar=0.0
      serr=0.0
      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
         avexp=avexp+expden(ind)
         sigexp=sigexp+expden(ind)**2
         avcal=avcal+calden(ind)
         sigcal=sigcal+calden(ind)**2
         covar=covar+expden(ind)*calden(ind)
         serr=serr+(expden(ind)-(calden(ind)*scale+shift))**2
         write(1,*) calden(ind)*scale+shift,expden(ind)
      enddo
      close(1)
      avexp=avexp/real(np)
      sigexp=sqrt(sigexp/real(np)-avexp**2)
      avcal=avcal/real(np)
      sigcal=sqrt(sigcal/real(np)-avcal**2)
      covar=covar/real(np)
      write(6,'(a,a,i8,f6.2)') 'Number of grid points, percentage of',
     &     ' total:',np,100.*real(np)/real(nx*ny*nz)
      write(6,'(a,a,2e14.6)') 'Average, sigma of calculated map ',
     &     '(masked region)   :',avcal,sigcal      
      write(6,'(a,a,2e14.6)') 'Average, sigma of experimental map ',
     &     '(masked region) :',avexp,sigexp
      write(6,'(a,a,f10.5)') 'Covariance  between calculated and',
     &     ' experimental map:',covar-avexp*avcal
      write(6,'(a,a,f10.5)') 'Correlation between calculated and',
     &     ' experimental map:',(covar-avexp*avcal)/(sigexp*sigcal)
      write(6,'(a,3e14.5)') 'Global  shift,scale,errorsum :',shift,
     &     scale,serr
      write(6,*)
#     if defined (OPTIMISE_OCCUPANCIES)
C----
C---- calculate optimal occupancies
C----
      call getocc(calden,expden,imin,imax,jmin,jmax,kmin,kmax,occ,
     &     maxat,maxmap,maxgrid,dim,dim2,ppi,ppj,ppk,ppc,np,nat,shift,
     &     scale)
      
      avocc=0.0
      do i=1,nat
         avocc=avocc+occ(i)
      enddo
      avocc=avocc/real(nat)
      write(6,'(a,f10.5)') 'Average occupancy:',avocc
C----
C---- write out pdb file with these occupancies
C----
      open(12,file='mapstat.pdb',status='unknown')
      do i=1,nat
         dum1=atnam(i)
         call gvsshl(dum1)
         if (gvflen(dum1).lt.4) then
            write(cdum1,'(a1,a3)') ' ',dum1(1:3)
         else
            cdum1=dum1(1:4)
         endif
         write (12,1000) i,cdum1,resnam(i)(1:3),chainid(i),
     &        resnr(i),xx(i),yy(i),zz(i),occ(i),Bfac(i)
      enddo
      write(12,'(a)') 'END'
      close(12)
C----
C---- recalculate densities with these shift, scale, occupancies
C----
      call calc_dens3(den1,den2,id,a1,a2,b1,b2,xx,yy,zz,nat,imin,imax,
     &     jmin,jmax,kmin,kmax,np,calden,space,orig,nx,ny,nz,maxat,
     &     maxmap,maxgrid,ppi,ppj,ppk,ppc,occ,Bfac,scale,shift)
C---- 
C---- recalculate mask statistics between calculated and experimental map
C----
      open(1,file='calexp_occ.den',status='unknown')
      avexp=0.0
      sigexp=0.0
      avcal=0.0
      sigcal=0.0
      covar=0.0
      serr=0.0
      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
         avexp=avexp+expden(ind)
         sigexp=sigexp+expden(ind)**2
         avcal=avcal+calden(ind)
         sigcal=sigcal+calden(ind)**2
         covar=covar+expden(ind)*calden(ind)
         serr=serr+(expden(ind)-calden(ind))**2
         write(1,*) calden(ind),expden(ind)
      enddo
      close(1)
      avexp=avexp/real(np)
      sigexp=sqrt(sigexp/real(np)-avexp**2)
      avcal=avcal/real(np)
      sigcal=sqrt(sigcal/real(np)-avcal**2)
      covar=covar/real(np)
      write(6,'(a)') 'Overall statistics (applying occupancies)'
      write(6,'(a,a,i8,f6.2)') 'Number of grid points, percentage of',
     &     ' total:',np,100.*real(np)/real(nx*ny*nz)
      write(6,'(a,a,2e14.6)') 'Average, sigma of calculated map ',
     &     '(masked region)   :',avcal,sigcal      
      write(6,'(a,a,2e14.6)') 'Average, sigma of experimental map ',
     &     '(masked region) :',avexp,sigexp
      write(6,'(a,a,f10.5)') 'Covariance  between calculated and',
     &     ' experimental map:',covar-avexp*avcal
      write(6,'(a,a,f10.5)') 'Correlation between calculated and',
     &     ' experimental map:',(covar-avexp*avcal)/(sigexp*sigcal)
      write(6,*)
#    endif
C----
C---- calculate backbone statistics
C----
#     if defined (OPTIMISE_OCCUPANCIES)
      write(6,'(a)') 'Backbone statistics (applying occupancies)'
#     else
      write(6,'(a)') 'Backbone statistics'
#     endif
      natd=0
      avocc=0.0
      do i=1,nat
         if (atnam(i).eq.'    N'.or.atnam(i).eq.'   CA'.or.
     &        atnam(i).eq.'    C'.or.atnam(i).eq.'    O') then
            natd=natd+1
            xd(natd)=xx(i)
            yd(natd)=yy(i)
            zd(natd)=zz(i)
            idd(natd)=id(i)
            den1d(natd)=den1(i)
            den2d(natd)=den2(1)
            avocc=avocc+occ(i)
            occd(natd)=occ(i)
            Bfacd(natd)=Bfac(i)
         endif
      enddo
      write(6,'(a,i5)') 'Number of backbone atoms:',natd
      avocc=avocc/real(natd)
#     if defined (OPTIMISE_OCCUPANCIES)
         write(6,'(a,f10.5)') 'Average occupancy:',avocc
#     endif

      call calc_dens3(den1d,den2d,idd,a1,a2,b1,b2,xd,yd,zd,natd,imin,
     &     imax,jmin,jmax,kmin,kmax,np,calden,space,orig,nx,ny,nz,
     &     maxat,maxmap,maxgrid,ppi,ppj,ppk,ppc,occd,Bfacd,scale,
     &     shift)

      avexp=0.0
      sigexp=0.0
      avcal=0.0
      sigcal=0.0
      covar=0.0
      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
         avcal=avcal+calden(ind)
         sigcal=sigcal+calden(ind)**2
         avexp=avexp+expden(ind)
         sigexp=sigexp+expden(ind)**2
         covar=covar+expden(ind)*calden(ind)
      enddo
      avexp=avexp/real(np)
      sigexp=sqrt(sigexp/real(np)-avexp**2)
      avcal=avcal/real(np)
      sigcal=sqrt(sigcal/real(np)-avcal**2)
      covar=covar/real(np)
      write(6,'(a,a,i8,f6.2)') 'Number of grid points, percentage of',
     &     ' total:',np,100.*real(np)/real(nx*ny*nz)
      write(6,'(a,a,2e14.6)') 'Average, sigma of calculated map ',
     &     '(masked region)   :',avcal,sigcal      
      write(6,'(a,a,2e14.6)') 'Average, sigma of experimental map ',
     &     '(masked region) :',avexp,sigexp
      write(6,'(a,a,f10.5)') 'Covariance  between calculated and',
     &     ' experimental map:',covar-avexp*avcal

      write(6,'(a,a,f10.5)') 'Correlation between calculated and',
     &     ' experimental map:',(covar-avexp*avcal)/(sigexp*sigcal)

      write(6,*)
C----
C---- calculate side chain statistics
C----
#     if defined (OPTIMISE_OCCUPANCIES)
      write(6,'(a)') 'Side chain statistics (applying occupancies)'
#     else
      write(6,'(a)') 'Side chain statistics'
#     endif
      natd=0
      avocc=0.0
      do i=1,nat
         if (atnam(i).ne.'    N'.and.atnam(i).ne.'   CA'.and.
     &        atnam(i).ne.'    C'.and.atnam(i).ne.'    O') then
            natd=natd+1
            xd(natd)=xx(i)
            yd(natd)=yy(i)
            zd(natd)=zz(i)
            idd(natd)=id(i)
            den1d(natd)=den1(i)
            den2d(natd)=den2(1)
            avocc=avocc+occ(i)
            occd(natd)=occ(i)
            Bfacd(natd)=Bfac(i)
         endif
      enddo
      write(6,'(a,i5)') 'Number of side chain atoms:',natd
      avocc=avocc/real(natd)
#     if defined (OPTIMISE_OCCUPANCIES)
         write(6,'(a,f10.5)') 'Average occupancy:',avocc
#     endif

      call calc_dens3(den1d,den2d,idd,a1,a2,b1,b2,xd,yd,zd,natd,imin,
     &     imax,jmin,jmax,kmin,kmax,np,calden,space,orig,nx,ny,nz,
     &     maxat,maxmap,maxgrid,ppi,ppj,ppk,ppc,occd,Bfacd,
     &     scale,shift)

      avexp=0.0
      sigexp=0.0
      avcal=0.0
      sigcal=0.0
      covar=0.0
      do n=1,np
         i=ppi(n)
         j=ppj(n)
         k=ppk(n)
         ind=ind3(i,j,k,dim,dim2)
         avexp=avexp+expden(ind)
         sigexp=sigexp+expden(ind)**2
         avcal=avcal+calden(ind)
         sigcal=sigcal+calden(ind)**2
         covar=covar+expden(ind)*calden(ind)
      enddo
      avexp=avexp/real(np)
      sigexp=sqrt(sigexp/real(np)-avexp**2)
      avcal=avcal/real(np)
      sigcal=sqrt(sigcal/real(np)-avcal**2)
      covar=covar/real(np)

      write(6,'(a,a,i8,f6.2)') 'Number of grid points, percentage of',
     &     ' total:',np,100.*real(np)/real(nx*ny*nz)
      write(6,'(a,a,2e14.6)') 'Average, sigma of calculated map ',
     &     '(masked region)   :',avcal,sigcal      
      write(6,'(a,a,2e14.6)') 'Average, sigma of experimental map ',
     &     '(masked region) :',avexp,sigexp
      write(6,'(a,a,f10.5)') 'Covariance  between calculated and',
     &     ' experimental map:',covar-avexp*avcal

      write(6,'(a,a,f10.5)') 'Correlation between calculated and',
     &     ' experimental map:',(covar-avexp*avcal)/(sigexp*sigcal)


 1000 format ('ATOM  ',i5,1x,a4,1x,a3,1x,a1,i4,4x,3f8.3,2f6.2)
 2000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,2f6.2)

      end      
