c23456789012345678901234567890123456789012345678901234567890123456789012
C----
C---- Routines for performing real-space refinement
C----
C---- (C) Bert de Groot, Goettingen, 2000
C----
      subroutine forrealrefi(expden,id,b1,b2,den1,den2,x,nat,space,orig,
     &     nx,ny,nz,verbflg,Bfac,shift,nasu,f,covar,ene,kreal)

      implicit none

      include 'params.f'
      
      integer maxgrid
      
      parameter(maxgrid=100)

      integer i,j,k,l,n,nat,id(*),np,ppi(maxat,maxgrid),nasu,natasu,
     &     ppj(maxat,maxgrid),ppk(maxat,maxgrid),imin,imax,jmin,dim,ind,
     &     jmax,kmin,kmax,nd(maxat),nx,ny,nz,orig(3),ind3,dim2
      
      real den1(*),b1(*),b2(*),pi,expden(*),calden(maxmap),
     &     Bfac(*),x(*),den2(*),avexp,avcal,covar,xd(3),xx(maxat),
     &     yy(maxat),zz(maxat),space(3),ene,minfourpisq,kreal,
     &     dden(maxmap),f(*),dum,rmsexp,sigexp,sigcal,shift(3),
     &     ppc(maxat,maxgrid)

      logical verbflg

      save calden
c----
c---- initialise
c----
      dim=nx
      dim2=nx*ny
      natasu=nat/nasu
      if (natasu.gt.maxat) then
         write(0,'(a)') 'Error - maxat exceeded'
         stop
      endif
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)
      do i=1,natasu
         xx(i)=(x(3*i-2)-shift(1))*10.0
         yy(i)=(x(3*i-1)-shift(2))*10.0
         zz(i)=(x(3*i)-shift(3))*10.0
      enddo
c----
c---- calculate densities from coordinates
c----
      call calc_dens(den1,den2,id,b1,b2,xx,yy,zz,natasu,
     &     imin,imax,jmin,jmax,kmin,kmax,nd,np,ppi,ppj,ppk,ppc,
     &     calden,space,orig,nx,ny,nz,maxat,maxmap,maxgrid,avcal,
     &     sigcal,Bfac)
c---- 
c---- covariance + differences between calculated and experimental densities
c----
      avexp=0.0
      sigexp=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  avexp=avexp+expden(ind)
                  sigexp=sigexp+expden(ind)**2
               endif
            enddo
         enddo
      enddo
      avexp=avexp/real(np)
      rmsexp=sqrt(sigexp/real(np))
      sigexp=sqrt(sigexp/real(np)-avexp**2)
      covar=0.0
      ene=0.0
      do k=kmin,kmax
         do j=jmin,jmax
            do i=imin,imax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  covar=covar+expden(ind)*calden(ind)
                  dden(ind)=(expden(ind)-avexp)/sigexp-
     &                 (calden(ind)-avcal)/sigcal
                  ene=ene+dden(ind)**2
               endif
            enddo
         enddo
      enddo
      ene=ene*kreal

      covar=(covar/real(np)-avexp*avcal)/(sigexp*sigcal)
c      covar=covar/real(np)
      
      if (verbflg) then
         write(6,'(a,f10.5,a,e12.5)') 'covar = ',covar,', ene = ',ene
         call flush(6)
      endif
C---- 
C---- calculate forces
C---- 
      do l=1,natasu
         do n=1,nd(l)
            i=ppi(l,n)
            j=ppj(l,n)
            k=ppk(l,n)
            xd(1)=real(i+orig(1)-1)*space(1)
            xd(2)=real(j+orig(2)-1)*space(2)
            xd(3)=real(k+orig(3)-1)*space(3)
            dum=2.0*kreal*ppc(l,n)*dden(ind3(i,j,k,dim,dim2))
            f(3*l-2)=f(3*l-2)+dum*(xd(1)-xx(l))
            f(3*l-1)=f(3*l-1)+dum*(xd(2)-yy(l))
            f(3*l)=f(3*l)+dum*(xd(3)-zz(l))
         enddo
      enddo
c----
c---- don't forget to put calden to zero here
c---- we do it here because here we remember which are the non-zero elements
c---- this saves us from re-initialising the whole thing which would be costly
c----
      do l=1,natasu
         do n=1,nd(l)
            calden(ind3(ppi(l,n),ppj(l,n),ppk(l,n),dim,dim2))=0.0
         enddo
      enddo

      do i=1,natasu
         x(3*i-2)=xx(i)/10.0+shift(1)
         x(3*i-1)=yy(i)/10.0+shift(2)
         x(3*i)=zz(i)/10.0+shift(3)
      enddo

      return

      end

      subroutine calc_dens(den1,den2,id,b1,b2,x,y,z,nat,
     &     imin,imax,jmin,jmax,kmin,kmax,nd,np,ppi,ppj,ppk,ppc,
     &     calden,space,orig,nx,ny,nz,maxat,maxmap,maxgrid,avcal,
     &     sigcal,Bfac)

C----
C---- calculates densities from coordinates
C----
      implicit none

      integer maxmap,maxat,maxgrid

      integer i,j,k,l,nat,id(maxat),np,ppi(maxat,maxgrid),
     &     imin,imax,jmin,jmax,kmin,kmax,ppj(maxat,maxgrid),ind,
     &     ppk(maxat,maxgrid),margin,nx,ny,nz,iimin,iimax,jjmin,
     &     jjmax,nd(maxat),kkmin,kkmax,orig(3),notinmap,ind3,dim,dim2
      
      real den1(maxat),b1(*),b2(*),pi,avcal,sigcal,
     &     calden(maxmap),minfourpisq,crit,rmscal,Bfac(*),
     &     x(maxat),y(maxat),z(maxat),dis,den2(maxat),xd(3),
     &     xx,yy,zz,space(3),contrib,cutoff,ppc(maxat,maxgrid),
     &     dd1,dd2,db1,db2

      logical bfirst, inmap
      data bfirst /.true./
      save bfirst
C----
C---- initialise
C---- cutoff sets the radius (in A) around each atom that is taken into account
C---- 2.5 A was empirically derived as a satisfactory compromise between
C---- accuracy and performance
C----
      dim=nx
      dim2=nx*ny
      cutoff=2.5
      margin=int(cutoff/min(space(1),space(2),space(3)))
      crit=cutoff**2
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)

      if (bfirst) then
c----
c---- initialise calden. Be aware that calden should be statically declared
c---- in the calling routine and that it should be re-initialised there also
c----
         do i=1,nx*ny*nz
            calden(i)=0.0
         enddo
c----
c---- check if atom coordinates are inside map volume
c---- 
         notinmap=0
         do l=1,nat
            inmap=.true.
            i=nint(x(l)/space(1))-orig(1)+1
            if (i.lt.1.or.i.gt.nx) inmap=.false.
            i=nint(y(l)/space(2))-orig(2)+1
            if (i.lt.1.or.i.gt.ny) inmap=.false.
            i=nint(z(l)/space(3))-orig(3)+1
            if (i.lt.1.or.i.gt.nz) inmap=.false.
            if (.not.inmap) notinmap=notinmap+1
         enddo
         if (notinmap.eq.nat) then
            write(0,'(a,a)') 'Error: atom coordinates do not ',
     &           'match map'
            stop
         endif
         if (notinmap.gt.0) then
            write(0,'(a,i5)') 'Warning : nr of atoms outside map:',
     &           notinmap
         endif
         bfirst=.false.
      endif
#     if defined (DEBUG)
         do i=1,nx
            do j=1,ny
               do k=1,nz
                  if (abs(calden(ind3(i,j,k,dim,dim2))).gt.1.0e-10) then
                     write (0,*) 'calden not zero:',
     &                    i,j,k,calden(ind3(i,j,k,dim,dim2))
                     stop
                  endif
               enddo
            enddo
         enddo
#     endif
C----
C---- create densities using the map parameters
C----
      iimin=1000
      jjmin=1000
      kkmin=1000
      iimax=-1000
      jjmax=-1000
      kkmax=-1000
      do l=1,nat
         nd(l)=0
         imin=nint(x(l)/space(1))-margin-orig(1)+1
         imin=max(1,imin)
         imax=nint(x(l)/space(1))+margin-orig(1)+1
         imax=min(nx,imax)
         jmin=nint(y(l)/space(2))-margin-orig(2)+1
         jmin=max(1,jmin)
         jmax=nint(y(l)/space(2))+margin-orig(2)+1
         jmax=min(ny,jmax)
         kmin=nint(z(l)/space(3))-margin-orig(3)+1
         kmin=max(1,kmin)
         kmax=nint(z(l)/space(3))+margin-orig(3)+1
         kmax=min(nz,kmax)
c         write(31,*) imin,imax,jmin,jmax,kmin,kmax
         xx=x(l)
         yy=y(l)
         zz=z(l)
         dd1=den1(l)
         dd2=den2(l)
         db1=1.0/(b1(id(l)+1)+Bfac(l))
         db2=1.0/(b2(id(l)+1)+Bfac(l))
         do k=kmin,kmax
            do j=jmin,jmax
               do i=imin,imax
                  xd(1)=real(i+orig(1)-1)*space(1)
                  xd(2)=real(j+orig(2)-1)*space(2)
                  xd(3)=real(k+orig(3)-1)*space(3)
                  dis=(xd(1)-xx)**2
                  dis=dis+(xd(2)-yy)**2
                  dis=dis+(xd(3)-zz)**2
                  if (dis.lt.crit) then
                     dis=dis*minfourpisq
                     contrib=dd1*exp(dis*db1)+dd2*exp(dis*db2)
c                     write(33,*) sqrt(dis),contrib
                     iimin=min(iimin,i)
                     jjmin=min(jjmin,j)
                     kkmin=min(kkmin,k)
                     iimax=max(iimax,i)
                     jjmax=max(jjmax,j)
                     kkmax=max(kkmax,k)
                     ind=ind3(i,j,k,dim,dim2)
                     calden(ind)=calden(ind)+contrib
                     nd(l)=nd(l)+1
                     if (nd(l).gt.maxgrid) then
                        write(0,'(a)') 'Error - maxgrid exceeded'
                        stop
                     endif
                     ppi(l,nd(l))=i
                     ppj(l,nd(l))=j
                     ppk(l,nd(l))=k
                     ppc(l,nd(l))=contrib
                  endif
               enddo
            enddo
         enddo
      enddo
C----
C----calculate statistics and normalise
C----
      np=0
      avcal=0.0
      sigcal=0.0
      do k=kkmin,kkmax
         do j=jjmin,jjmax
            do i=iimin,iimax
               ind=ind3(i,j,k,dim,dim2)
               if (calden(ind).gt.1.0e-5) then
                  np=np+1
                  avcal=avcal+calden(ind)
                  sigcal=sigcal+calden(ind)**2
               else
                  calden(ind)=0.0
               endif
            enddo
         enddo
      enddo
      avcal=avcal/real(np)
      rmscal=sqrt(sigcal/real(np))
      sigcal=sqrt(sigcal/real(np)-avcal**2)
c      write(6,*) 'avcal,sigcal,rmscal:',avcal,sigcal,rmscal

      imin=iimin
      imax=iimax
      jmin=jjmin
      jmax=jjmax
      kmin=kkmin
      kmax=kkmax

c      write(6,*) 'np=',np,' ntot=',((kmax-kmin)+1)*((jmax-jmin)+1)*
c     &     ((imax-imin)+1)
         
      return

      end

      subroutine forcalcmap(id,b1,b2,den1,den2,x,nat,space,orig,
     &     nx,ny,nz,Bfac,nasu,calden,rot_asu,trans_asu)
C----
C---- driver routine for calc_map
C---- calculate densities for whole unit cell without applying symmetry
C----
C---- in theory, applying symmetry could speed things up. in practice however..
C----
      implicit none

      include 'params.f'
      
      integer maxgrid
      
      parameter(maxgrid=100)

      integer i,j,k,l,m,n,nat,id(*),np,ppi(maxat,maxgrid),nasu,natasu,
     &     ppj(maxat,maxgrid),ppk(maxat,maxgrid),dim,ind,nd(maxat),
     &     nx,ny,nz,orig(3),ind3,dim2,ii,jj,kk,ind3c
      
      real den1(*),b1(*),b2(*),calden(*),Bfac(*),x(*),den2(*),
     &     xx(maxat),yy(maxat),zz(maxat),space(3),ppc(maxat,maxgrid),
     &     rot_asu(*),trans_asu(*)

      logical bFirst
      data bFirst /.true./
      save ppi,ppj,ppk,nd,bFirst
c----
c---- initialise
c----
      dim=nx
      dim2=nx*ny
      natasu=nat/nasu
      if (nat.gt.maxat) then
         write(0,'(a)') 'Error - maxat exceeded'
         stop
      endif
      do i=1,nat
         xx(i)=x(3*i-2)*10.0
         yy(i)=x(3*i-1)*10.0
         zz(i)=x(3*i)*10.0
      enddo
      if (bFirst) then
         bFirst=.false.
      else
         do l=1,nat
            do n=1,nd(l)
               calden(ind3(ppi(l,n),ppj(l,n),ppk(l,n),dim,dim2))=0.0
            enddo
         enddo
      endif
c----
c---- calculate densities from coordinates, whole unit cell
c----
      call calc_map(den1,den2,id,b1,b2,xx,yy,zz,nat,nd,np,ppi,ppj,
     &     ppk,ppc,calden,space,orig,nx,ny,nz,maxat,maxgrid,Bfac)

      return

      end

      subroutine forcalcmapsym(id,b1,b2,den1,den2,x,nat,space,orig,
     &     nx,ny,nz,Bfac,nasu,calden,rot_asu,trans_asu)
C----
C---- driver routine for calc_map
C---- calculate densities for first ASU and apply symmetry for contributions
C---- of other ASU's
C----
C---- in theory, applying symmetry could speed things up. in practice however..
C----

      implicit none

      include 'params.f'
      
      integer maxgrid
      
      parameter(maxgrid=100)

      integer i,j,k,l,m,n,nat,id(*),np,ppi(maxat,maxgrid),nasu,natasu,
     &     ppj(maxat,maxgrid),ppk(maxat,maxgrid),dim,ind,nd(maxat),
     &     nx,ny,nz,orig(3),ind3,dim2,ii,jj,kk,ind3c
      
      real den1(*),b1(*),b2(*),calden(*),Bfac(*),x(*),den2(*),
     &     xx(maxat),yy(maxat),zz(maxat),space(3),ppc(maxat,maxgrid),
     &     rot_asu(*),trans_asu(*)

      logical bFirst
      data bFirst /.true./
      save ppi,ppj,ppk,nd,bFirst
c----
c---- initialise
c----
      dim=nx
      dim2=nx*ny
      natasu=nat/nasu
      if (natasu.gt.maxat) then
         write(0,'(a)') 'Error - maxat exceeded'
         stop
      endif
      do i=1,natasu
         xx(i)=x(3*i-2)*10.0
         yy(i)=x(3*i-1)*10.0
         zz(i)=x(3*i)*10.0
      enddo
      if (bFirst) then
         do m=1,nasu
            write(0,*) 'ASU:',m
            write(0,*) rot_asu(ind3c(m,1,1,3,9)),
     &           rot_asu(ind3c(m,2,1,3,9)),
     &           rot_asu(ind3c(m,3,1,3,9))
            write(0,*) rot_asu(ind3c(m,1,2,3,9)),
     &           rot_asu(ind3c(m,2,2,3,9)),
     &           rot_asu(ind3c(m,3,2,3,9))
            write(0,*) rot_asu(ind3c(m,1,3,3,9)),
     &           rot_asu(ind3c(m,2,3,3,9)),
     &           rot_asu(ind3c(m,3,3,3,9))
             write(0,*) trans_asu(3*m-2),trans_asu(3*m-1),trans_asu(3*m)
         enddo
         bFirst=.false.
      else
         do l=1,natasu
            do n=1,nd(l)
               calden(ind3(ppi(l,n),ppj(l,n),ppk(l,n),dim,dim2))=0.0
            enddo
         enddo
         do m=2,nasu
            do l=1,natasu
               do n=1,nd(l)
                  i=ppi(l,n)
                  j=ppj(l,n)
                  k=ppk(l,n)
                  ii=(i-1)*rot_asu(ind3c(m,1,1,3,9))+
     &                 (j-1)*rot_asu(ind3c(m,2,1,3,9))+
     &                 (k-1)*rot_asu(ind3c(m,3,1,3,9))+1
                  jj=(i-1)*rot_asu(ind3c(m,1,2,3,9))+
     &                 (j-1)*rot_asu(ind3c(m,2,2,3,9))+
     &                 (k-1)*rot_asu(ind3c(m,3,2,3,9))+1
                  kk=(i-1)*rot_asu(ind3c(m,1,3,3,9))+
     &                 (j-1)*rot_asu(ind3c(m,2,3,3,9))+
     &                 (k-1)*rot_asu(ind3c(m,3,3,3,9))+1
                  ii=ii+int(trans_asu(3*m-2)*10.)
                  jj=jj+int(trans_asu(3*m-1)*10.)
                  kk=kk+int(trans_asu(3*m)*10.)
                  ii=mod(ii,nx)
                  if (ii.lt.1) ii=ii+nx
                  jj=mod(jj,ny)
                  if (jj.lt.1) jj=jj+ny
                  kk=mod(kk,nz)
                  if (kk.lt.1) kk=kk+nz
                  ind=ind3(ii,jj,kk,dim,dim2)
                  calden(ind)=0.0
               enddo
            enddo
         enddo
      endif
c----
c---- calculate densities from coordinates, first asu only
c----
      call calc_map(den1,den2,id,b1,b2,xx,yy,zz,natasu,nd,np,ppi,ppj,
     &     ppk,ppc,calden,space,orig,nx,ny,nz,maxat,maxgrid,Bfac)

C----
C---- apply symops to add contribution from other asu's
C----
      do m=2,nasu
         do l=1,natasu
            do n=1,nd(l)
               i=ppi(l,n)
               j=ppj(l,n)
               k=ppk(l,n)
               ii=(i-1)*rot_asu(ind3c(m,1,1,3,9))+
     &            (j-1)*rot_asu(ind3c(m,2,1,3,9))+
     &            (k-1)*rot_asu(ind3c(m,3,1,3,9))+1
               jj=(i-1)*rot_asu(ind3c(m,1,2,3,9))+
     &            (j-1)*rot_asu(ind3c(m,2,2,3,9))+
     &            (k-1)*rot_asu(ind3c(m,3,2,3,9))+1
               kk=(i-1)*rot_asu(ind3c(m,1,3,3,9))+
     &            (j-1)*rot_asu(ind3c(m,2,3,3,9))+
     &            (k-1)*rot_asu(ind3c(m,3,3,3,9))+1
               ii=ii+int(trans_asu(3*m-2)*10.)
               jj=jj+int(trans_asu(3*m-1)*10.)
               kk=kk+int(trans_asu(3*m)*10.)
               ii=mod(ii,nx)
               if (ii.lt.1) ii=ii+nx
               jj=mod(jj,ny)
               if (jj.lt.1) jj=jj+ny
               kk=mod(kk,nz)
               if (kk.lt.1) kk=kk+nz
               ind=ind3(ii,jj,kk,dim,dim2)
               calden(ind)=calden(ind)+ppc(l,n)
            enddo
         enddo
      enddo

      return

      end

      subroutine calc_map(den1,den2,id,b1,b2,x,y,z,nat,nd,np,ppi,ppj,
     &     ppk,ppc,calden,space,orig,nx,ny,nz,maxat,maxgrid,Bfac)
C----
C---- calculates densities from coordinates
C---- clone from calc_dens for fft use
C----
      implicit none

      integer maxat,maxgrid

      integer i,j,k,l,nat,id(*),np,ppi(maxat,maxgrid),
     &     imin,imax,jmin,jmax,kmin,kmax,ppj(maxat,maxgrid),ind,
     &     ppk(maxat,maxgrid),margin,nx,ny,nz,nd(maxat),orig(3),
     &     notinmap,ind3,dim,dim2,ii,jj,kk
      
      real den1(maxat),b1(*),b2(*),pi,
     &     calden(*),minfourpisq,crit,rmscal,Bfac(*),
     &     x(maxat),y(maxat),z(maxat),dis,den2(maxat),xd(3),
     &     xx,yy,zz,space(3),contrib,cutoff,ppc(maxat,maxgrid),
     &     dd1,dd2,db1,db2

      logical bfirst, inmap
      data bfirst /.true./
      save bfirst
C----
C---- initialise
C---- cutoff sets the radius (in A) around each atom that is taken into account
C---- 2.5 A was empirically derived as a satisfactory compromise between
C---- accuracy and performance
C----
      dim=nx
      dim2=nx*ny
      cutoff=2.5
      margin=int(cutoff/min(space(1),space(2),space(3)))
      crit=cutoff**2
      pi=4.0*atan(1.0)
      minfourpisq=-(4.*pi*pi)

      if (bfirst) then
c----
c---- initialise calden. Be aware that calden should be statically declared
c---- in the calling routine and that it should be re-initialised there also
c----
         do i=1,nx*ny*nz
            calden(i)=0.0
         enddo
         bfirst=.false.
      endif
C----
C---- create densities using the map parameters
C---- apply PBC such that all atoms and corresponding density fall inside map
C----
      do l=1,nat
         nd(l)=0
         imin=nint(x(l)/space(1))-margin-orig(1)+1
         imax=nint(x(l)/space(1))+margin-orig(1)+1
         jmin=nint(y(l)/space(2))-margin-orig(2)+1
         jmax=nint(y(l)/space(2))+margin-orig(2)+1
         kmin=nint(z(l)/space(3))-margin-orig(3)+1
         kmax=nint(z(l)/space(3))+margin-orig(3)+1
         xx=x(l)
         yy=y(l)
         zz=z(l)
         dd1=den1(l)
         dd2=den2(l)
         db1=1.0/(b1(id(l)+1)+Bfac(l))
         db2=1.0/(b2(id(l)+1)+Bfac(l))
         do k=kmin,kmax
            kk=mod(k,nz)
            if (kk.lt.1) kk=kk+nz
            do j=jmin,jmax
               jj=mod(j,ny)
               if (jj.lt.1) jj=jj+ny
               do i=imin,imax
                  ii=mod(i,nx)
                  if (ii.lt.1) ii=ii+nx
                  xd(1)=real(i-1)*space(1)
                  xd(2)=real(j-1)*space(2)
                  xd(3)=real(k-1)*space(3)
                  dis=(xd(1)-xx)**2
                  dis=dis+(xd(2)-yy)**2
                  dis=dis+(xd(3)-zz)**2
                  if (dis.lt.crit) then
                     dis=dis*minfourpisq
                     contrib=dd1*exp(dis*db1)+dd2*exp(dis*db2)
                     ind=ind3(ii,jj,kk,dim,dim2)
                     calden(ind)=calden(ind)+contrib
                     nd(l)=nd(l)+1
                     if (nd(l).gt.maxgrid) then
                        write(0,'(a)') 'Error - maxgrid exceeded'
                        stop
                     endif
                     ppi(l,nd(l))=ii
                     ppj(l,nd(l))=jj
                     ppk(l,nd(l))=kk
                     ppc(l,nd(l))=contrib
                  endif
               enddo
            enddo
         enddo
      enddo
         

c      write(0,'(a)') 'Writing output map...' 
c      open(22,file='xx.xpl',status='unknown')
c      write(22,*)
c      write(22,'(i8)') 1
c      write(22,'(a)') 'REMARKS map generated by fft'
c      write(22,'(9i8)') 96,0,95,96,0,95,100,0,99
c      write(22,'(1p,6e12.5)') 96.0,96.0,100.0,90.0,90.0,90.0
c      write(22,'(a)') 'ZYX'
c      do k=1,nz
c         write(22,'(i8)') k
c         write(22,'(6e12.5)') ((calden((k-1)*dim2+(j-1)*dim+i),
c     &        i=1,nx),j=1,ny)
c      enddo
c      close(22)


      return

      end

      integer function ind3 (i, j, k, dim, dim2)
C----
C---- returns index a of 1-dimensional x(a) for the corresponding
C---- (equivalenced) 3-dimensional x(i,j,k)
C----

      implicit none
      
      integer i,j,k,dim,dim2

      ind3 = (k-1)*dim2+(j-1)*dim+i 

      return

      end
      integer function ind3c (i, j, k, dim, dim2)
C----
C---- returns index a of 1-dimensional x(a) for the corresponding
C---- 3-dimensional x[i,j,k] in C
C----

      implicit none
      
      integer i,j,k,dim,dim2

      ind3c = (i-1)*dim2+(j-1)*dim+k 

      return

      end

      subroutine forreadxplormap(nx,ny,nz,space,orig,den)
      
      implicit none

      integer i,j,k,ntit,nx,ny,nz,ngrid(3),orig(3),dim,dim2,den(*),
     &     gvflen

      real space(3),cell(6)

      character*255 buf

      logical gvfisf
C----
C---- read in experimental densities, Xplor formatted format
C----
      write(6,'(a)') 'Reading map...'
      call getenv('REALREFI',buf)
      call gvstcp(buf)
      if (gvfisf(buf)) then
         open(1,file=buf,status='old')
      else
         call gvstcp(buf)
         write(0,'(a,a)') 'Error -  file not found : ',
     &        buf(1:gvflen(buf))
         stop
      endif
      read(1,'(a255)') buf
      read(1,*) ntit
      do i=1,ntit
         read(1,'(a255)') buf
      enddo
      read(1,'(9i8)') 
     +  ngrid(1),orig(1),nx,
     +  ngrid(2),orig(2),ny,
     +  ngrid(3),orig(3),nz
      nx=nx-orig(1)+1
      ny=ny-orig(2)+1
      nz=nz-orig(3)+1
      read(1,'(1p,6e12.5)') (cell(i),i=1,6)
      if (cell(4).ne.90.0.or.cell(5).ne.90.0.or.cell(6).ne.90.0) then
         write(0,'(a)') 'Only orthogonal cells are supported. Sorry.'
         write(0,'(a)') 'Use e.g. the CCP4 program maprot to generate ',
     &        'one from your map.'
         stop
      endif
      space(1)=cell(1)/ngrid(1)
      space(2)=cell(2)/ngrid(2)
      space(3)=cell(3)/ngrid(3)
      read(1,'(a255)') buf
C----
C---- this trick allows den to be 1-dimensional and therefore we
C---- can be more memory-efficient (faster too!)
C---- den(i,j,k) is now den((k-1)*dim2+(j-1)*dim+i)
C---- (this would btw also be the result of equivalencing the two)
C----
      dim=nx
      dim2=nx*ny
      do k=1,nz
         read(1,'(a255)') buf
         read(1,'(6e12.5)') ((den((k-1)*dim2+(j-1)*dim+i),
     &        i=1,nx),j=1,ny)
      enddo
      close(1)

      return

      end
      subroutine forreadxplormapheader(nx,ny,nz,space,orig)
      
      implicit none

      integer i,ntit,nx,ny,nz,ngrid(3),orig(3),gvflen

      real space(3),cell(6)

      logical gvfisf

      character*255 buf
C----
C---- read in map header, Xplor formatted format
C----
      write(6,'(a)') 'Reading map header...'
      call getenv('REALREFI',buf)
      call gvstcp(buf)
      if (gvfisf(buf)) then
         open(1,file=buf,status='old')
      else
         call gvstcp(buf)
         write(0,'(a,a)') 'Error -  file not found : ',
     &        buf(1:gvflen(buf))
         stop
      endif
      read(1,'(a255)') buf
      read(1,*) ntit
      do i=1,ntit
         read(1,'(a255)') buf
      enddo
      read(1,'(9i8)') 
     +  ngrid(1),orig(1),nx,
     +  ngrid(2),orig(2),ny,
     +  ngrid(3),orig(3),nz
      nx=nx-orig(1)+1
      ny=ny-orig(2)+1
      nz=nz-orig(3)+1
      read(1,'(1p,6e12.5)') (cell(i),i=1,6)
      if (cell(4).ne.90.0.or.cell(5).ne.90.0.or.cell(6).ne.90.0) then
         write(0,'(a)') 'Only orthogonal cells are supported. Sorry.'
         write(0,'(a)') 'Use e.g. the CCP4 program maprot to generate ',
     &        'one from your map.'
         stop
      endif
      space(1)=cell(1)/ngrid(1)
      space(2)=cell(2)/ngrid(2)
      space(3)=cell(3)/ngrid(3)
      write(6,'(a)') 'Map parameters:'
      write(6,'(a,3f8.3)') 'Cell axes  :',(cell(i),i=1,3)
      write(6,'(a,3f8.3)') 'Cell angles:',(cell(i),i=4,6)
      write(6,'(a,3i8)') 'Grid       :',(ngrid(i),i=1,3)
      write(6,'(a,3i8)') 'Extent     :',nx,ny,nz
      write(6,'(a,3i8)') 'Origin     :',(orig(i),i=1,3)
      write(6,'(a,3f8.3)') 'Spacing    :',(space(i),i=1,3)

      close(1)

      return

      end
      SUBROUTINE GVSTCP (TEXT)
C///
C-------------------------------------------------------------------------------
C----
C---- SHIFTS ALL CHARACTERS LEFTWARDS IN TEXT UNTIL NO BLANKS ARE LEFT.     ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT       NONE
      CHARACTER*(*)  TEXT
      INTEGER        GVFLEN, LENTXT, ICHR, ILOOP
C----
C---- SKIP EMPTY TEXT
C----
      IF (GVFLEN(TEXT).LE.1) RETURN
C----
C---- SET THE START LENGTH
C----
      LENTXT=GVFLEN(TEXT)
C----
C---- CHECK IF THERE ARE ANY CHARACTERS
C----
      IF (LENTXT.LT.2) RETURN
C----
C---- START WITH A LEFT SHIFT
C----
      CALL GVSSHL (TEXT)
C----
C---- LOOP OVER TEXT AND FIND BLANKS
C----
      ICHR=1
   10 LENTXT=GVFLEN(TEXT)
      IF (LENTXT.LT.2) RETURN
C----
C---- LOOP OVER THE STRING TILL FIRST BLANK IS FOUND, THEN SHIFT DOWN REST
C----
      DO 30 ICHR=1,LENTXT-1
         IF (TEXT(ICHR:ICHR).EQ.' ') THEN
C----------
C---------- SHIFT ALL HIGHER CHARS ONE DOWN, AND REDO THE TEST
C----------
            DO 20 ILOOP=ICHR,LENTXT-1
               TEXT(ILOOP:ILOOP)=TEXT(ILOOP+1:ILOOP+1)
   20       CONTINUE
            TEXT(LENTXT:LENTXT)=' '
            GOTO 10
         END IF
   30 CONTINUE

      END

      INTEGER FUNCTION GVFLEN(TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- RETURNS THE LENGTH OF TEXT                                            ----
C---- THE LENGTH IS DEFINED AS THE SEQUENCE NUMBER IN TEXT OF THE LAST      ----
C---- NON-BLANK CHARACTER. UNDEFINED CHARACTERS ARE ASSUMED TO BE BLANK.    ----
C----                                                                       ----
C---- TEXT IS A CHARACTER STRING OF ANY LENGTH                              ----
C---- DO NOT FORGET TO DECLARE GVFLEN AS AN INTEGER IN THE CALLING ROUTINE  ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT         NONE
      CHARACTER*(*)    TEXT
      INTEGER          I, ICH
C----
C---- INITIATE
C----
      GVFLEN=0
C----
C---- MEASURE THE LENGTH
C----
      DO 10 I=LEN(TEXT),1,-1
         ICH=ICHAR(TEXT(I:I))
         IF (ICH.NE.ICHAR(' ').AND.ICH.NE.0) THEN
            GVFLEN=I
            RETURN
         END IF
   10 CONTINUE

      RETURN
      END

      SUBROUTINE GVSSHL (TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A LEFT SHIFT IS PERFORMED ON TEXT UNTIL ALL LEADING BLANCKS ARE GONE  ----
C---- TEXT SHOULD BE A CHARACTER STRING OF UNLIMITTED LENGTH                ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT      NONE
      CHARACTER*(*) TEXT
      INTEGER       GVFLEN, I, J, LENTXT
C----
C---- DETERMINE THE LENGTH OF THE TEXT
C----
      LENTXT=GVFLEN(TEXT)
      IF (LENTXT.GT.0) THEN
C-------
C------- IF THERE ARE LEADING BLANKS, SHIFT TO THE LEFT
C-------
         DO 30 I=1,LENTXT
            IF (ICHAR(TEXT(I:I)).NE.ICHAR(' ')) THEN
               IF (I.EQ.1) RETURN
               DO 10 J=I,LENTXT
                  TEXT(J-I+1:J-I+1)=TEXT(J:J)
   10          CONTINUE
               DO 20 J=LENTXT-I+2,LENTXT
                  TEXT(J:J)=' '
   20          CONTINUE
               RETURN
            END IF
   30    CONTINUE
      END IF

      RETURN
      END
      SUBROUTINE GVSSHR (TEXT,LENGTH)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A RIGHT SHIFT IS PERFORMED ON TEXT UNTIL ALL TRAILING BLANKS ARE GONE ----
C---- TEXT SHOULD BE A CHARACTER STRING OF LENGTH LENGTH.                   ----
C----                                                                       ----
C---- STRANGE ERRORS WILL OCCUR IF TEXT IS NOT PROPERLY DEFINED IN THE      ----
C---- CALLING MODULE. CHANCES ARE THAT THESE ERRORS SHOW UP IN A TOTALLY    ----
C---- DIFFERENT WAY THEN YOU WOULD EXPECT.                                  ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT      NONE
      INTEGER       LENGTH, I
      CHARACTER*(*) TEXT
C----
C---- LOOK FOR TRIVIAL ERRORS
C----
      IF (LENGTH.LT.0) THEN
         WRITE(0,*) ('Length of input text is < zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.0) THEN
         WRITE(0,*) ('Length of input text is zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.1) THEN
         WRITE(0,*) ('Length of input text is 1 in GVSSHR')
         RETURN
      END IF
C----
C---- IF THE TEXT IS OK, SEE IF IT ENDS AT BLANK
C----
   10 IF (TEXT(LENGTH:LENGTH).NE.' ') RETURN
C----
C---- IF THERE ARE TRAILING BLANKS, SHIFT EVERYTHING UP BY ONE, AND TEST AGAIN
C----
      DO 20 I=LENGTH,2,-1
         TEXT(I:I)=TEXT(I-1:I-1)
   20 CONTINUE
      TEXT(1:1)=' '
      GOTO 10

      END

      LOGICAL FUNCTION GVFISF(FILNAM)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- BECOMES TRUE IF THE FILE FILNAM EXISTS.                               ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT          NONE
      CHARACTER*(*)  FILNAM
      INTEGER        GVFLEN

C-------
C------- TEST FOR EMPTY FILE NAME
C-------
         IF (GVFLEN(FILNAM).EQ.0) THEN
            GVFISF=.FALSE.
            RETURN
         END IF
C-------
C------- SEE IF THE FILE EXISTS
C-------
         INQUIRE (FILE=FILNAM,EXIST=GVFISF)

      RETURN
      END
