      program rottrans
c23456789012345678901234567890123456789012345678901234567890123456789012
C----
C---- (C) Bert de Groot, Goettingen 2000
C----
C---- The functionality of this program is very similar
C---- to the ESSENS program from the RAVE package of Gerard Kleywegt.
C---- The main difference is the applied scoring function.
C---- 
      implicit none

      include 'params.f'

      integer i,j,k,l,m,n,nat,atnr(maxat),resnr(maxat),imin,imax,jmin,
     &     jmax,kmin,kmax,u,v,w,nskipped,percout,nrot,nrotdone,nm,mm,
     &     mpi(maxmask),mpj(maxmask),mpk(maxmask),nx,ny,nz,orig(3),
     &     ngrid(3),kk,kkmax,ntit,np,ppi(maxgrid),ppj(maxgrid),
     &     ppk(maxgrid),id(maxat),iargc,offset,dim,dim2,ppn(maxgrid),
     &     nrotreal,nres,iproj,mpp(maxmask),gvflen
      
      character*5 atnam(maxat),resnam(maxat)

      real pi,den(maxmap),x(maxat*3),cell(6),xav(3),alpha,beta,gamma,
     &     sa,ca,sb,cb,sg,cg,xr(maxat),yr(maxat),zr(maxat),denscutoff,
     &     space(3),smax,score,rscore(7,maxscore+1),a1(10),a2(10),
     &     b1(10),b2(10),ppd(maxgrid),den1(maxat),den2(maxat),avexp,
     &     sigexp,ppe(maxgrid),avcal,sigcal,refaxis(3),ip,ri,rj,rk,
     &     kstep,rotstep,Bfac(maxat),cutoff,occ(maxat),shift,scale,
     &     proj,minproj,maxproj

      real*8 c(3,3),d(3),e(3)

      character*1 chainid(maxat)

      character*4 cdum1,cdum2

      character*255 buf
      
      character*80 sdum

      logical had(maxscore+1),gvfisf

#if defined(DISTANCE)
      if (iargc().ne.5) then
         write(0,'(a,a)') 'Usage: rottrans in.map in.mask out.scores ',
     &        'shift scale < in.pdb '
# else
      if (iargc().ne.3) then
         write(0,'(a,a)') 'Usage: rottrans in.map in.mask out.scores ',
     &        '< in.pdb '
#endif
         write(0,*)
         write(0,'(a,a)') 'Performs a rigid body search of "in.pdb" ',
     &        'through the map'
         write(0,'(a,a)') 'for the best-fitting orientations within ',
     &        'the mask.'
         write(0,'(a)') 'in.map has to be x-plor formatted map for now'
         write(0,'(a,a)') 'masks can be made based on .pdb files with ',
     &        'pdb2mask'
#if defined(DISTANCE)
         write(0,'(a,a)') 'Values for shift and scale can be obtained',
     &        ' by running "mapstat"'
#endif
         stop
      endif

      rotstep=10.0
      cutoff=2.5
      pi=4.0*atan(1.0)
      nm=0
      nskipped=0
      denscutoff=0.0
      do i=1,maxscore+1
         had(i)=.false.
         rscore(7,i)=-1000.0
      enddo
C----
C---- count the number of rotations to be done
C----
      nrot=0
      do ri=0.0,359.9,rotstep
         do rj=0.0,179.9,rotstep
            beta=pi*rj/180.0
            rk=nint((360.0/rotstep-1.0)*sin(beta)+1.0)
            kstep=360.0/rk
            do rk=0.0,359.9,kstep
               nrot=nrot+1
            enddo
         enddo
      enddo
      write(6,'(a,i10)') 'Nr of rotations to be done:',nrot
      percout=nrot/100
C----
C---- read the input structure
C----
      nat=0
      nres=1
      cdum2='xxxx'
 50   continue
      read(5,'(a80)',err=60,end=70) sdum
      if ((sdum(1:4).ne.'ATOM').and.(sdum(1:4).ne.'HETA')) goto 50
      nat=nat+1
      if (nat.gt.maxat) then
         write(0,'(a)') 'Error - MAXAT exceeded'
         stop
      endif
      read(sdum,1000) atnam(nat),resnam(nat),chainid(nat),cdum1,
     &     x(3*nat-2),x(3*nat-1),x(3*nat),occ(nat),Bfac(nat)
      if (nat.gt.1) then
         if (cdum1.ne.cdum2) nres=nres+1
      endif
      if (atnam(nat).ne.'     ') then
         call gvsshr(atnam(nat),5)
      else
         write(0,*) 'Encountered empty atom name for atom number',nat
         stop
      endif
      resnr(nat)=nres
      atnr(nat)=nat
      cdum2=cdum1
      goto 50
 60   write(0,'(a)') 'Error while reading structure from stdin'
      stop
 70   continue
      if (nat.eq.0) then
         write(0,'(a,a)') 'Unexpected end of file while reading ',
     &        'structure from stdin'
         stop
      endif
      write(6,'(a,i5)') 'Number of atoms: ',nat

      xav(1)=0.0
      xav(2)=0.0
      xav(3)=0.0

      do i=1,nat
         xav(1)=xav(1)+x(3*i-2)
         xav(2)=xav(2)+x(3*i-1)
         xav(3)=xav(3)+x(3*i)
      enddo

      xav(1)=xav(1)/real(nat)
      xav(2)=xav(2)/real(nat)
      xav(3)=xav(3)/real(nat)

      do i=1,nat
         x(3*i-2)=x(3*i-2)-xav(1)
         x(3*i-1)=x(3*i-1)-xav(2)
         x(3*i)=x(3*i)-xav(3)
      enddo
C----
C---- get the atomic scattering factors etc
C----
      call getatomscat(a1,a2,b1,b2,atnam,nat,Bfac,id,den1,den2)
      do i=1,nat
         den1(i)=den1(i)*occ(i)
         den2(i)=den2(i)*occ(i)
      enddo
C----
C---- read in experimental densities, Xplor formatted format
C----
      call getarg(1,sdum)
      call readxplormapheader(sdum,cell,nx,ny,nz,space,orig,ngrid)
      dim=nx
      dim2=nx*ny
      if (nx*ny*nz.gt.maxmap) then
         write(0,'(a)') 'Error - nx,ny,nz too large.'
         write(0,'(a,i10)') 'Increase maxmap to at least',nx*ny*nz
         stop
      endif
      call readxplormap(sdum,nx,ny,nz,space,orig,den)
C----
C---- read mask
C---- mask points should contain coordinates (in grid points) of the 
C---- desired positions of the center of mass of the trial model
C----
      write(6,'(a)') 'Reading mask...'
      call getarg(2,sdum)
      if (gvfisf(sdum)) then
         open(1,file=sdum,status='old')
      else
         call gvstcp(sdum)
         write(0,'(a,a)') 'Error - file not found : ',
     &        sdum(1:gvflen(sdum))
         stop
      endif
      read(1,*,end=90) refaxis(1),refaxis(2),refaxis(3)
      write(6,'(a,3f8.3)') 'Reference axis: ',refaxis(1),refaxis(2),
     &     refaxis(3)
      do i=1,1e8
         nm=nm+1
         if (nm.gt.maxmask) then
            write(0,*) 'Nr of mask points too large. Increase maxmask'
            stop
         endif
         read(1,*,end=90) mpi(nm),mpj(nm),mpk(nm)
      enddo
 90   continue
      nm=nm-1
      close(1)
      write(6,'(a,i6)') 'Nr of mask points read:',nm
C----
C---- project all mask points onto principal axis and store 
C----
      maxproj=-1.e8
      minproj=1.e8
      do i=1,nm
         proj=refaxis(1)*real(mpi(i))+refaxis(2)*real(mpj(i))+
     &        refaxis(3)*real(mpk(i))
         minproj=min(minproj,proj)
         maxproj=max(maxproj,proj)
      enddo
c      write(6,*) 'minproj,maxproj',minproj,maxproj
      do i=1,nm
         proj=refaxis(1)*real(mpi(i))+
     &        refaxis(2)*real(mpj(i))+
     &        refaxis(3)*real(mpk(i))
         proj=(proj-minproj)/(maxproj-minproj)
         mpp(i)=nint(proj*real(maxscore)+1.0)
         write(6,*) i,mpp(i)
      enddo
C----
C---- generate the different rotations and translations
C----
      call getarg(3,sdum)
      open(1,file=sdum,status='unknown')
      nrotdone=0
      nrotreal=0
C----
C---- get shift and scale to be applied to calculated densities
C----
#if defined(DISTANCE)
      call getarg(4,sdum)
      read(sdum,*) shift
      call getarg(5,sdum)
      read(sdum,*) scale
      write(6,'(a,2e15.6)') 'Shift, scale:',shift,scale
#endif
C----
C---- loop over rotations
C----
      do ri=0.0,359.9,rotstep
         gamma=pi*ri/180.0
         cg=cos(gamma)
         sg=sin(gamma)
         do rj=0.0,179.9,rotstep
            beta=pi*rj/180.0
            cb=cos(beta)
            sb=sin(beta)
C----
C---- for the last angle we need a different grid if we use Euler angles
C----
            rk=nint((360.0/rotstep-1.0)*sin(beta)+1.0)
            kstep=360.0/rk
            do rk=0.0,359.9,kstep
               alpha=pi*rk/180.0
               ca=cos(alpha)
               sa=sin(alpha)
               do l=1,nat
                  xr(l)=x(3*l-2)*(cg*cb*ca-sg*sa)
                  xr(l)=xr(l)-x(3*l-1)*(sg*cb*ca+cg*sa)
                  xr(l)=xr(l)+x(3*l)*(sb*ca)
                  yr(l)=x(3*l-2)*(cg*cb*sa+sg*ca)
                  yr(l)=yr(l)-x(3*l-1)*(sg*cb*sa-cg*ca)
                  yr(l)=yr(l)+x(3*l)*(sb*sa)
                  zr(l)=x(3*l-2)*(-cg*sb)
                  zr(l)=zr(l)+x(3*l-1)*(sg*sb)
                  zr(l)=zr(l)+x(3*l)*cb
               enddo
C----
C---- check if the principal axis for this rotation fulfills the axis
C---- criterion. First set up 3x3 covariance matrix for the coordinates.
C----
               do n=1,3
                  do m=1,3
                     c(n,m)=0.0
                  enddo
               enddo
               do l=1,nat
                  c(1,1)=c(1,1)+xr(l)*xr(l)
                  c(1,2)=c(1,2)+xr(l)*yr(l)
                  c(1,3)=c(1,3)+xr(l)*zr(l)
                  c(2,1)=c(2,1)+yr(l)*xr(l)
                  c(2,2)=c(2,2)+yr(l)*yr(l)
                  c(2,3)=c(2,3)+yr(l)*zr(l)
                  c(3,1)=c(3,1)+zr(l)*xr(l)
                  c(3,2)=c(3,2)+zr(l)*yr(l)
                  c(3,3)=c(3,3)+zr(l)*zr(l)
               enddo
C---- 
C---- diagonalise
C---- 
               call ql77(3,c,d,e,3)
C----
C---- compare the primary axis to the reference axis
C---- (cos(20 degrees) = 0.94)
C----
               ip=c(1,3)*refaxis(1)
               ip=ip+c(2,3)*refaxis(2)
               ip=ip+c(3,3)*refaxis(3)
               if (abs(ip).lt.0.5) goto 200
C----
C---- calculate densities for this rotation
C----
               call calc_dens2(den1,den2,id,a1,a2,b1,b2,xr,yr,zr,cutoff,
     &              nat,imin,imax,jmin,jmax,kmin,kmax,np,ppi,ppj,ppk,
     &              ppd,avcal,sigcal,space,nx,ny,nz,Bfac)

               do l=1,np
                  ppn(l)=ppi(l)+ppj(l)*dim+ppk(l)*dim2
               enddo
C----
C---- loop over all shifts in mask (translations)
C----
               do mm=1,nm
C----
C---- make sure this translation has all atoms on grid
C----
                  score=0.0
                  u=mpi(mm)
                  if ((imin+u).lt.1.or.(imax+u).gt.nx) goto 100
                  v=mpj(mm)
                  if ((jmin+v).lt.1.or.(jmax+v).gt.ny) goto 100
                  w=mpk(mm)
                  if ((kmin+w).lt.1.or.(kmax+w).gt.nz) goto 100
                  iproj=mpp(mm)
                  had(iproj)=.true.
C----
C---- density cut-off for center of mass
C----
                  offset=(w-1)*dim2+(v-1)*dim+u
c                  if (den(offset).lt.denscutoff) 
c     &                 goto 100
C----
C---- calculate the score for this orientation
C----
#if defined (DISTANCE)
                  do l=1,np
                     ppe(l)=den(offset+ppn(l))
                     score=score+(ppe(l)-(ppd(l)*scale+shift))**2
                  enddo
                  score=1.0/(score/float(np))
#else
                  avexp=0.0
                  sigexp=0.0
                  do l=1,np
                     ppe(l)=den(offset+ppn(l))
                     avexp=avexp+ppe(l)
                     sigexp=sigexp+ppe(l)**2
                     score=score+ppe(l)*ppd(l)
                  enddo
                  avexp=avexp/float(np)
                  sigexp=sqrt(sigexp/float(np) - avexp**2)
                  score=score/float(np)
                  score=(score-avexp*avcal)/(sigexp*sigcal)
#endif
                  call update_scores(iproj,ri,rj,rk,u,v,w,score,
     &                 rscore,maxscore)
c                  call update_n_scores(ri,rj,rk,u,v,w,score,rscore,
c     &                 maxscore)
 100              continue
                  if (abs(score).lt.1.0e-6) nskipped=nskipped+1
               enddo
               nrotreal=nrotreal+1
 200           continue
               nrotdone=nrotdone+1
c               if (nrotdone.gt.500) stop
               if (mod(nrotdone,percout).eq.0) then
                  write (6,'(a,f6.2)') 
     &              'Percentage done: ',real(int(100.0*nrotdone/nrot)+1)
                  call flush(6)
               endif
            enddo
         enddo 
      enddo

      do i=1,maxscore
         if (had(i))
     &   write(1,'(6f8.3,e20.10)') rscore(1,i),rscore(2,i),rscore(3,i),
     &        rscore(4,i),rscore(5,i),rscore(6,i),rscore(7,i)
      enddo
               
      close(1)

      write(6,'(a,f6.2)') 'Percentage of translations skipped:',
     &     100.0*real(nskipped)/(real(nm)*real(nrot))

      write(6,'(a,f6.2)') 'Percentage of rotations skipped...:',
     &     100.0-100.0*real(nrotreal)/real(nrot)


 1000 format (12x,a4,1x,a4,a1,a4,4x,3f8.3,2f6.2)

      end      
