      SUBROUTINE GVSTCP (TEXT)
C///
C-------------------------------------------------------------------------------
C----
C---- SHIFTS ALL CHARACTERS LEFTWARDS IN TEXT UNTIL NO BLANKS ARE LEFT.     ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT       NONE
      CHARACTER*(*)  TEXT
      INTEGER        GVFLEN, LENTXT, ICHR, ILOOP
C----
C---- SKIP EMPTY TEXT
C----
      IF (GVFLEN(TEXT).LE.1) RETURN
C----
C---- SET THE START LENGTH
C----
      LENTXT=GVFLEN(TEXT)
C----
C---- CHECK IF THERE ARE ANY CHARACTERS
C----
      IF (LENTXT.LT.2) RETURN
C----
C---- START WITH A LEFT SHIFT
C----
      CALL GVSSHL (TEXT)
C----
C---- LOOP OVER TEXT AND FIND BLANKS
C----
      ICHR=1
   10 LENTXT=GVFLEN(TEXT)
      IF (LENTXT.LT.2) RETURN
C----
C---- LOOP OVER THE STRING TILL FIRST BLANK IS FOUND, THEN SHIFT DOWN REST
C----
      DO 30 ICHR=1,LENTXT-1
         IF (TEXT(ICHR:ICHR).EQ.' ') THEN
C----------
C---------- SHIFT ALL HIGHER CHARS ONE DOWN, AND REDO THE TEST
C----------
            DO 20 ILOOP=ICHR,LENTXT-1
               TEXT(ILOOP:ILOOP)=TEXT(ILOOP+1:ILOOP+1)
   20       CONTINUE
            TEXT(LENTXT:LENTXT)=' '
            GOTO 10
         END IF
   30 CONTINUE

      END

      INTEGER FUNCTION GVFLEN(TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- RETURNS THE LENGTH OF TEXT                                            ----
C---- THE LENGTH IS DEFINED AS THE SEQUENCE NUMBER IN TEXT OF THE LAST      ----
C---- NON-BLANK CHARACTER. UNDEFINED CHARACTERS ARE ASSUMED TO BE BLANK.    ----
C----                                                                       ----
C---- TEXT IS A CHARACTER STRING OF ANY LENGTH                              ----
C---- DO NOT FORGET TO DECLARE GVFLEN AS AN INTEGER IN THE CALLING ROUTINE  ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT         NONE
      CHARACTER*(*)    TEXT
      INTEGER          I, ICH
C----
C---- INITIATE
C----
      GVFLEN=0
C----
C---- MEASURE THE LENGTH
C----
      DO 10 I=LEN(TEXT),1,-1
         ICH=ICHAR(TEXT(I:I))
         IF (ICH.NE.ICHAR(' ').AND.ICH.NE.0) THEN
            GVFLEN=I
            RETURN
         END IF
   10 CONTINUE

      RETURN
      END

      SUBROUTINE GVSSHL (TEXT)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A LEFT SHIFT IS PERFORMED ON TEXT UNTIL ALL LEADING BLANCKS ARE GONE  ----
C---- TEXT SHOULD BE A CHARACTER STRING OF UNLIMITTED LENGTH                ----
C----                                                                       ----
C---- THIS ROUTINE IS WRITTEN IN FORTRAN 77.                                ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT      NONE
      CHARACTER*(*) TEXT
      INTEGER       GVFLEN, I, J, LENTXT
C----
C---- DETERMINE THE LENGTH OF THE TEXT
C----
      LENTXT=GVFLEN(TEXT)
      IF (LENTXT.GT.0) THEN
C-------
C------- IF THERE ARE LEADING BLANKS, SHIFT TO THE LEFT
C-------
         DO 30 I=1,LENTXT
            IF (ICHAR(TEXT(I:I)).NE.ICHAR(' ')) THEN
               IF (I.EQ.1) RETURN
               DO 10 J=I,LENTXT
                  TEXT(J-I+1:J-I+1)=TEXT(J:J)
   10          CONTINUE
               DO 20 J=LENTXT-I+2,LENTXT
                  TEXT(J:J)=' '
   20          CONTINUE
               RETURN
            END IF
   30    CONTINUE
      END IF

      RETURN
      END
      SUBROUTINE GVSSHR (TEXT,LENGTH)
C+++
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- A RIGHT SHIFT IS PERFORMED ON TEXT UNTIL ALL TRAILING BLANKS ARE GONE ----
C---- TEXT SHOULD BE A CHARACTER STRING OF LENGTH LENGTH.                   ----
C----                                                                       ----
C---- STRANGE ERRORS WILL OCCUR IF TEXT IS NOT PROPERLY DEFINED IN THE      ----
C---- CALLING MODULE. CHANCES ARE THAT THESE ERRORS SHOW UP IN A TOTALLY    ----
C---- DIFFERENT WAY THEN YOU WOULD EXPECT.                                  ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C===
      IMPLICIT      NONE
      INTEGER       LENGTH, I
      CHARACTER*(*) TEXT
C----
C---- LOOK FOR TRIVIAL ERRORS
C----
      IF (LENGTH.LT.0) THEN
         WRITE(0,*) ('Length of input text is < zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.0) THEN
         WRITE(0,*) ('Length of input text is zero in GVSSHR')
         RETURN
      ELSE IF (LENGTH.EQ.1) THEN
         WRITE(0,*) ('Length of input text is 1 in GVSSHR')
         RETURN
      END IF
C----
C---- IF THE TEXT IS OK, SEE IF IT ENDS AT BLANK
C----
   10 IF (TEXT(LENGTH:LENGTH).NE.' ') RETURN
C----
C---- IF THERE ARE TRAILING BLANKS, SHIFT EVERYTHING UP BY ONE, AND TEST AGAIN
C----
      DO 20 I=LENGTH,2,-1
         TEXT(I:I)=TEXT(I-1:I-1)
   20 CONTINUE
      TEXT(1:1)=' '
      GOTO 10

      END

      LOGICAL FUNCTION GVFISF(FILNAM)
C///
C-------------------------------------------------------------------------------
C----                                                                       ----
C---- BECOMES TRUE IF THE FILE FILNAM EXISTS.                               ----
C----                                                                       ----
C---- THIS FUNCTION IS WRITTEN IN FORTRAN 77.                               ----
C----                                                                       ----
C-------------------------------------------------------------------------------
C\\\
      IMPLICIT          NONE
      CHARACTER*(*)  FILNAM
      INTEGER        GVFLEN

C-------
C------- TEST FOR EMPTY FILE NAME
C-------
         IF (GVFLEN(FILNAM).EQ.0) THEN
            GVFISF=.FALSE.
            RETURN
         END IF
C-------
C------- SEE IF THE FILE EXISTS
C-------
         INQUIRE (FILE=FILNAM,EXIST=GVFISF)

      RETURN
      END
